@@ -6,7 +6,7 @@ set -e
 echo ""
 echo "*** Testing ncgen and ncdump using some test CDL files."
 echo "*** creating tst_small.nc from ref_tst_small.cdl..."
-../ncgen/ncgen -o tst_small.nc $srcdir/ref_tst_small.cdl
+../ncgen/ncgen -b -o tst_small.nc $srcdir/ref_tst_small.cdl
 echo "*** creating tst_small.cdl from tst_small.nc..."
 ./ncdump tst_small.nc > tst_small.cdl
 diff -w tst_small.cdl $srcdir/ref_tst_small.cdl
