@@ -7,6 +7,7 @@
 
 static const unsigned int MAX_UINT = 0xffffffff;
 
+static OCerror occomputeskipdatar(OCstate*, OCnode*, ocoffset_t offset);
 static int mergedas1(OCnode* dds, OCnode* das);
 static int converttype(OCtype etype, char* value, char* memory);
 static char* pathtostring(OClist* path, char* separator, int usecdfname);
@@ -14,7 +15,7 @@ static void computefullname(OCnode* node);
 
 /* Process ocnodes to fix various semantic issues*/
 void
-computeocsemantics(OClist* ocnodes)
+occomputesemantics(OClist* ocnodes)
 {
     unsigned int i;
     OCASSERT((ocnodes != NULL));
@@ -28,14 +29,14 @@ computeocsemantics(OClist* ocnodes)
 }
 
 void
-computeocfullnames(OCnode* root)
+occomputefullnames(OCnode* root)
 {
     unsigned int i;
     if(root->name != NULL) computefullname(root);
     if(root->subnodes != NULL) { /* recurse*/
         for(i=0;i<oclistlength(root->subnodes);i++) {
 	    OCnode* node = (OCnode*)oclistget(root->subnodes,i);
-	    computeocfullnames(node);
+	    occomputefullnames(node);
 	}
     }
 }
@@ -49,7 +50,7 @@ computefullname(OCnode* node)
 
     OCASSERT((node->name != NULL));
     path = oclistnew();
-    collectpathtonode(node,path);
+    occollectpathtonode(node,path);
     tmp = pathtostring(path,PATHSEPARATOR,1);
     if(tmp == NULL) {
         fullname = nulldup(node->name);
@@ -88,15 +89,15 @@ pathtostring(OClist* path, char* separator, int usecdfname)
 
 /* Collect the set of nodes ending in "node"*/
 void
-collectpathtonode(OCnode* node, OClist* path)
+occollectpathtonode(OCnode* node, OClist* path)
 {
     if(node == NULL) return;
-    collectpathtonode(node->container,path);
+    occollectpathtonode(node->container,path);
     oclistpush(path,(ocelem)node);
 }
 
 OCnode*
-makeocnode(char* name, OCtype ptype, OCnode* root)
+ocmakenode(char* name, OCtype ptype, OCnode* root)
 {
     OCnode* cdf = (OCnode*)ocmalloc(sizeof(OCnode));
     MEMCHECK(cdf,(OCnode*)NULL);
@@ -235,93 +236,6 @@ fail:
     return 0;
 }
 
-/* For those nodes that are uniform in size, compute size
-   size of the node*/
-size_t
-ocsetsize(OCnode* node)
-{
-    size_t count, subnodesum;
-    unsigned int i;
-    int isscalar = (node->array.rank == 0);
-    size_t instancesize;
-    size_t dimsize; /* to give to parent*/
-
-    instancesize = 0; /* assume not uniform*/
-    dimsize = 0;
-
-    /* compute total # of elements if dimensioned*/
-    count = 1;
-    for(i=0;i<node->array.rank;i++) {
-	OCnode* dim = (OCnode*)oclistget(node->array.dimensions,i);
-	count *= (dim->dim.declsize);
-    }
-
-    /* Recursively compute sizes of subnodes, if any*/
-    subnodesum = 0;
-    if(node->subnodes != NULL) {
-	int nonuniform = 0;
-        for(i=0;i<oclistlength(node->subnodes);i++) {
-	    OCnode* subnode = (OCnode*)oclistget(node->subnodes,i);
-	    size_t subsize = ocsetsize(subnode); /* includes subnode dimension counts*/
-	    if(subsize == 0) nonuniform = 1;
-	    subnodesum += subsize;
-	}
-	if(nonuniform) subnodesum = 0;
-    }
-
-    switch (node->octype) {
-        case OC_Primitive:
-	    switch (node->etype) {
-	    case OC_String: case OC_URL: /* not uniform*/
-		instancesize = 0;
-		dimsize = 0; /* not uniform*/
-		break; /* not uniform*/
-	    case OC_Byte:
-	    case OC_UByte:
-	    case OC_Char:
-		instancesize = (isscalar?BYTES_PER_XDR_UNIT:1);
-	        dimsize = instancesize;
-		/* We have to watch out for the fact that packed instances have padding in the xdr packet*/
-		if(!isscalar) { /* padding to multiple of BYTE_PER_XDR_UNIT*/
-		    unsigned int rem;
-	            dimsize = count*instancesize;
-		    rem = (dimsize % BYTES_PER_XDR_UNIT);
-		    if(rem > 0) dimsize += (BYTES_PER_XDR_UNIT - rem);
-		    dimsize += 2*BYTES_PER_XDR_UNIT; /* the dimension counts (repeated)*/
-		}
-		break;
-	    case OC_Float64:
-	    case OC_Int64:
-	    case OC_UInt64:
-		instancesize = (2*BYTES_PER_XDR_UNIT); /*double = 2 xdr units*/
-	        dimsize = count*instancesize + (isscalar?0:2*BYTES_PER_XDR_UNIT);
-		break;
-	    default:
-		instancesize = (BYTES_PER_XDR_UNIT); /* all others: 1 xdr unit*/
-	        dimsize = count*instancesize + (isscalar?0:2*BYTES_PER_XDR_UNIT);
-		break;
-	    }
-	    break;
-
-        case OC_Sequence: /* never uniform, but instances may be*/
-	    dimsize = 0;
-	    instancesize = subnodesum;
-	    break;
-
-        case OC_Grid:
-	case OC_Dataset:
-        case OC_Structure:
-	    instancesize = subnodesum;
-	    dimsize = count*instancesize + (isscalar?0:BYTES_PER_XDR_UNIT);
-	    break;
-
-        default: OCPANIC1("ocmap: encountered unexpected node type: %x",node->octype);
-	    break;
-    }
-    node->dap.instancesize = instancesize;
-    node->dap.arraysize = dimsize;
-    return dimsize;
-}
 
 void
 ocfreeroot(OCnode* root)
@@ -352,18 +266,11 @@ ocfreetree(OCtree* tree)
     ocfree(tree->constraint);
     ocfree(tree->text);
     if(tree->data.xdrs != NULL) {
-        xdr_destroy(tree->data.xdrs);
-	ocfree(tree->data.xdrs);
+        xxdr_free(tree->data.xdrs);
     }
-#ifdef OC_DISK_STORAGE
-    if(tree->dxdclass == OCDATA) {
-	ocfree(tree->data.filename);
-        if(tree->data.file != NULL) fclose(tree->data.file);
-    }
-#else
-    ocfree(tree->data.xdrdata);
-#endif
-    freeocmemdata(tree->data.memdata);
+    ocfree(tree->data.filename); /* may be null */
+    if(tree->data.file != NULL) fclose(tree->data.file);
+    ocfree(tree->data.memory);
     ocfree(tree);
 }
 
@@ -531,7 +438,7 @@ mergedas1(OCnode* dds, OCnode* das)
 
 
 
-#if 0 /*def IGNORE*/
+#ifdef IGNORE
 
 int
 ocddsdasmerge(OCstate* state, OCnode* ddsroot, OCnode* dasroot)
@@ -691,3 +598,157 @@ occorrelate(OCnode* dds, OCnode* dxd)
     ocuncorrelate(dds);
     return occorrelater(dds,dxd);
 }
+
+/*
+Mark cacheable those primitive String/URL typed nodes
+that are contained only in structures with rank > 0.
+*/
+void
+ocmarkcacheable(OCstate* state, OCnode* ddsroot)
+{
+    int i,j;
+#ifdef IGNORE
+    int ok;
+#endif
+    OClist* treenodes = ddsroot->tree->nodes;
+    OClist* path = oclistnew();
+    for(i=0;i<oclistlength(treenodes);i++) {
+        OCnode* node = (OCnode*)oclistget(treenodes,i);
+	if(node->octype != OC_Primitive) continue;
+	if(node->etype != OC_String && node->etype != OC_URL) continue;
+	/* collect node path */
+        oclistclear(path);
+        occollectpathtonode(node,path);	
+#ifdef IGNORE
+        ok = 1;
+#endif
+	for(j=1;j<oclistlength(path)-1;j++) {/* skip top level dataset and node itself*/
+            OCnode* pathnode = (OCnode*)oclistget(path,j);
+	    if(pathnode->octype != OC_Structure
+		|| pathnode->array.rank > 0) {
+#ifdef IGNORE
+	    ok=0;
+#endif
+	    break;
+	    }
+	}	
+#ifdef IGNORE
+	if(ok) {
+   	    node->cache.cacheable = 1;
+	    node->cache.valid = 0;
+	}
+#endif
+    }
+    oclistfree(path);
+}
+
+
+/*
+Fill in the OCnode.skip fields
+*/
+OCerror
+occomputeskipdata(OCstate* state, OCnode* ddsroot)
+{
+    OCerror stat = OC_NOERR;
+    OCASSERT(ddsroot->octype == OC_Dataset);
+    stat = occomputeskipdatar(state,ddsroot,0);    
+    return stat;
+}
+
+/* Recursive helper for computeskipdata */
+static OCerror
+occomputeskipdatar(OCstate* state, OCnode* xnode, ocoffset_t offset)
+{
+    OCerror stat = OC_NOERR;
+    int i,nfields;
+    int scalar = 0;
+    ocoffset_t instancesize, totalsize;
+
+    scalar = (xnode->array.rank == 0 ? 1 : 0);
+
+    /* Set skip count and offset*/
+    if(xnode->octype == OC_Sequence)
+	xnode->skip.count = OCINDETERMINATE;
+    else
+        xnode->skip.count = totaldimsize(xnode);
+
+    xnode->skip.offset = offset; /* possibly overridden below */
+
+    switch (xnode->octype) {
+
+    case OC_Primitive:
+	switch(xnode->etype) {
+	case OC_String: case OC_URL:
+	    instancesize = OCINDETERMINATE;
+	    totalsize = OCINDETERMINATE;
+	    break;
+	case OC_Char: case OC_Byte: case OC_UByte:
+	    if(!scalar) {/*=>packed*/
+		instancesize = octypesize(xnode->etype);
+		totalsize = instancesize * xnode->skip.count;
+		totalsize = RNDUP(totalsize);
+		totalsize += 2*XDRUNIT; /* overhead is double count */
+		break;
+	    }
+	    /* !packed => singleton char object */
+	    /* fall thru */
+	case OC_Int16: case OC_UInt16:
+	case OC_Int32: case OC_UInt32:
+	case OC_Int64:	case OC_UInt64:
+	case OC_Float32: case OC_Float64:
+	    instancesize = octypesize(xnode->etype);
+	    instancesize = RNDUP(instancesize); /* make multiple of XDRUNIT */
+	    totalsize = (instancesize*xnode->skip.count); /* overhead is double count */
+	    if(!scalar)
+		totalsize += 2*XDRUNIT; /* overhead is double count */
+	    break;
+
+	default: OCPANIC("unexpected etype"); /* better not happen */
+	}
+	break;
+
+    case OC_Sequence:
+	offset = (xnode->skip.offset = OCINDETERMINATE); /* do not know field offsets for arbitrary record */
+    case OC_Dataset:
+    case OC_Grid:    
+    case OC_Structure:
+	/* Compute size of each field and sum */
+	nfields = oclistlength(xnode->subnodes);
+	instancesize = 0; /* of structure as a whole */
+	for(i=0;i<nfields;i++) {
+	    OCnode* subnode = (OCnode*)oclistget(xnode->subnodes,i);
+	    ocoffset_t fieldsize;
+	    if(offset == OCINDETERMINATE || instancesize == OCINDETERMINATE)
+	        stat = occomputeskipdatar(state,subnode,OCINDETERMINATE);
+	    else 
+                stat = occomputeskipdatar(state,subnode,offset+instancesize);
+	    if(stat != OC_NOERR) goto done;
+	    fieldsize = subnode->skip.totalsize;
+	    if(instancesize == OCINDETERMINATE || fieldsize == OCINDETERMINATE)
+		instancesize = OCINDETERMINATE;
+	    else
+		instancesize += fieldsize; 
+	}
+	if(instancesize != OCINDETERMINATE) {
+            instancesize = RNDUP(instancesize); /* make multiple of XDRUNIT */
+	    totalsize = (instancesize*xnode->skip.count); /* overhead is single count */
+	    if(!scalar)
+	        totalsize += XDRUNIT; /* overhead is single count */
+        } else {
+	    totalsize = OCINDETERMINATE; 
+	}
+	if(xnode->octype == OC_Sequence) {
+            totalsize = OCINDETERMINATE;
+            offset = OCINDETERMINATE;
+	}
+	break;
+
+    default: OCPANIC("unexpected octype"); /* better not happen */
+    }
+
+    xnode->skip.instancesize = instancesize;
+    xnode->skip.totalsize = totalsize;
+
+done:
+    return stat;
+}
