@@ -7,7 +7,10 @@
 /* Forward */
 struct OCstate;
 
+#define ocmax(x,y) ((x) > (y) ? (x) : (y))
+
 extern char* ocstrndup(const char* s, size_t len);
+extern int ocstrncmp(const char* s1, const char* s2, size_t len);
 
 extern size_t octypesize(OCtype etype);
 extern char*  octypetostring(OCtype octype);
@@ -15,7 +18,7 @@ extern char*  octypetoddsstring(OCtype octype);
 extern char* ocerrstring(int err);
 extern OCerror ocsvcerrordata(struct OCstate*,char**,char**,long*);
 extern OCerror octypeprint(OCtype etype, char* buf, size_t bufsize, void* value);
-extern size_t ocxdrsize(OCtype etype);
+extern size_t xxdrsize(OCtype etype);
 
 extern size_t totaldimsize(OCnode*);
 
@@ -23,12 +26,6 @@ extern void makedimlist(OClist* path, OClist* dims);
 
 extern int findbod(OCbytes* buffer, size_t*, size_t*);
 
-extern int xdr_skip(XDR* xdrs, unsigned int len);
-extern int xdr_skip_strings(XDR* xdrs, unsigned int n);
-extern unsigned int xdr_roundup(unsigned int n);
-
-extern unsigned int ocbyteswap(unsigned int i);
-
 /* Reclaimers*/
 extern void freeOCnode(OCnode*,int);
 extern void ocfreeprojectionclause(OCprojectionclause* clause);
